c Version 1.5.1 beta 2/3/96
c----------------------------------------------------------------------
c                    =========================
c                       common block EPRPRM
c                    =========================
c
c	Variable and common block definition for NLSL fitting of
c  experimental spectra using LBLL-family slow-motional calculations.
c
c
c IMPORTANT NOTES: 
c
c   The order in which the parameters appear in /eprprm/ is critical to 
c   the proper functioning of the NLSL program. If this common block is
c   to be changed, the following rules must be observed:
c
c   (1) To permit alias names to be used for the g,A, and diffusion
c       tensors, the parameters gxx,gyy,gzz,axx,ayy,azz,dx,dy,dz,wxx,wyy,wzz
c       should appear contiguously in that order in the common block.
c
c   (2) The parameters igsph,iasph,irsph,iwsph should appear contiguously
c       in that order in the common block. (See function tcheck in file
c       sphten)
c
c   (3) The order in which the parameters appear in /eprprm/ must
c       be consistent with the names defined in the block data section
c       of file "ipfind.f"
c
c   (4) The residence times tl,tkxy,tkzz have been replaced with the
c   	parameters pml, pmxy, pmzz. This is to facilitate fitting using
c	non-Brownian models where it is desirable to vary the residence
c	time and the diffusion rate constant together (i.e. keeping the
c       product, which is related to the rms jump angle, constant).
c
c   (5) The non-Brownian model flags have been changed to ml, mzz, and
c	mxy for perpendicular, parallel, and internal motions, respectively.
c       The flags may be set to 0, 1, and 2 for Brownian, free, and
c	jump diffusion, respectively.
c
c   (6) The old "wint" parameters have been changed to "gib", and
c       now the additional width refers to Gaussian inhomogeneous width
c       rather than an additional Lorentzian width
c
c   (7) A new spectral linewidth broadening parameter, "lb", has been
c       added
c
c***********************************************************************
c
	double precision gxx,gyy,gzz,axx,ayy,azz,wxx,wyy,wzz,dx,dy,dz,
     #         d0x,d0y,d0z,
     #         pml,pmxy,pmzz,djf,djfprp,oss,psi,ald,bed,gad,alm,bem,gam,
     #         c20,c22,c40,c42,c44,gib0,gib2,lb,b0,gamman,cgtol,shiftr,
     #         shifti,range,fldi,dfld,a0,g0,w0,expl,expkxy,expkzz,phase,
     #         dc20 
c
c     srls floating-point parameters
c
        double precision b10,b20,a20,a22,a40,a42,a44,bag,
     #         d0rt,d0pt,drt,dpt
c
	double precision faa,fgm,fam,fwm,fgd,fad,fwd,cpot,xlk
c
c     srsl floating-point parameters
c
        double precision d2psi,d2bed
c
c  ---  srls or non-srls logical variable
c
        logical choise
c
c  ---  srls basis set file <name>
c
        character*30 setnam
c
	integer	in2,ipdf,ist,ml,mxy,mzz,lemx,lomx,kmn,kmx,mmn,mmx,ipnmx,
     #          nort,igflg,iaflg,iwflg,irflg,idflg,nstep,ideriv,itype,
     #          ndim,
     #		nfld,jkmn,jmmn,ipt,itm,itd,ipsi0,lband,kband,ldelta,
     #          kdelta,lptmx,kptmx,neltot,nelv,nelre,nelim,ncgstp
c
c     srls integer parameters
c
        integer jmx,nmx,imx,jdelta
c
	common /eprprm/ phase,gib0,gib2,
     #         wxx,wyy,wzz,
     #         gxx,gyy,gzz,
     #         axx,ayy,azz,
     #         dx,dy,dz,
     #         d0x,d0y,d0z,
     #         pml,pmxy,pmzz,
     #         djf,djfprp,oss,
     #         psi,ald,bed,gad,alm,bem,gam,
     #         c20,c22,c40,c42,c44,lb,dc20,
     #         b10,b20,a20,a22,a40,a42,a44,bag,
     #         b0,gamman,cgtol,shiftr,shifti,range,fldi,dfld,
     #         a0,g0,w0,expl,expkxy,expkzz,faa(5),fgm(5),fwm(5),
     #         fam(2,5),fgd(2,5),fad(2,5),fwd(2,5),cpot(5,5),xlk(5,5),
     #         d0rt,d0pt,drt,dpt,
     #         d2psi(5,5),d2bed(5,5),
     #	       in2,ipdf,ist,ml,mxy,mzz,lemx,lomx,kmn,kmx,mmn,mmx,ipnmx,
     #         nort,nstep,nfld,ideriv,jmx,nmx,imx,
     #         iwflg,igflg,iaflg,irflg,idflg,ndim,
     #         jkmn,jmmn,itype,ipt,itm,itd,ipsi0,lband,kband,
     #         ldelta,kdelta,lptmx,kptmx,neltot,nelv,nelre,nelim,
     #         ncgstp,jdelta
c
c  ---  common block for the big choise
c
        common /choise/ choise
        common /setnam/ setnam
c
c *** The following constants identify the position of several
c     important parameters within the fepr (and fparm) arrays.
c     THESE CONSTANTS *MUST* BE REDEFINED IF THE PARAMETER ORDER 
c     IN EPRPRM IS CHANGED!!!
c
c     integer IPHASE,IGIB0,IGIB2,IGXX,IGZZ,IAXX,IAZZ,IWXX,IWZZ,
c    #        IDX,IDZ,ID0X,ID0Z,IPML,IPMXY,IPMZZ,IDJF,IDJFPRP,IOSS,
c    #        IPSI,IALD,IBED,
c    #        IGAD,IALM,IGAM,IC20,IC44,ILB,IB0,IGAMAN,IML,ILEMX,IIPDF,
c    #        IIST,IIN2,IIGFLG,IIAFLG,IIWFLG,IIRFLG,IIDFLG,INORT,
c    #        INSTEP,ISHIFT,
c    #        ICGTOL,IFLDI,IDFLD,INFLD,IIDERV,INDIM,IDC20,IRANGE,
c    #        IB10,IB20,IA20,IA44,IBAG,IJMX
c
      integer  IPHASE,IGIB0,IGIB2,IWXX,IWZZ,IGXX,IGZZ,
     #         IAXX,IAZZ,IDX,IDZ,ID0X,ID0Z,IPML,
     #         IPMXY,IPMZZ,
     #         IDJF,IDJFPRP,IOSS,IPSI,IALD,IBED,
     #         IGAD,IALM,IGAM,IC20,IC44,ILB,IDC20,
     #         IB10,IB20,IA20,IA44,IBAG,
     #         IB0,IGAMAN,ICGTOL,ISHIFT,IRANGE,
     #         IFLDI,IDFLD,
     #         IIN2,IIPDF,IIST,IML,ILEMX,INORT,INSTEP,
     #         INFLD,IIDERV,IJMX,IIWFLG,IIGFLG,
     #         IIAFLG,IIRFLG,IIDFLG,INDIM
c
      parameter(IPHASE=1,IGIB0=2,IGIB2=3,IWXX=4,IWZZ=6,IGXX=7,IGZZ=9,
     #         IAXX=10,IAZZ=12,IDX=13,IDZ=15,ID0X=16,ID0Z=18,IPML=19,
     #         IPMXY=20,IPMZZ=21,
     #         IDJF=22,IDJFPRP=23,IOSS=24,IPSI=25,IALD=26,IBED=27,
     #         IGAD=28,IALM=29,IGAM=31,IC20=32,IC44=36,ILB=37,IDC20=38,
     #         IB10=39,IB20=40,IA20=41,IA44=45,IBAG=46,
     #         IB0=47,IGAMAN=48,ICGTOL=49,ISHIFT=50,IRANGE=52,
     #         IFLDI=53,IDFLD=54,
     #         IIN2=1,IIPDF=2,IIST=3,IML=4,ILEMX=7,INORT=14,INSTEP=15,
     #         INFLD=16,IIDERV=17,IJMX=18,IIWFLG=21,IIGFLG=22,
     #         IIAFLG=23,IIRFLG=24,IIDFLG=25,INDIM=26)
c
c    ISHIFT or ISHIFTR?





