c Based on MTSL - Version 1.4  5/18/93
c----------------------------------------------------------------------
c                    =========================
c                         subroutine MTSS
c                    =========================
c
c   Given an array representing the maximum projection of each basis
c   vector on the solution vector (array basis), the number of elements
c   (ndim) and a minimum projection defining the level at which a basis
c   vector is to be considered significant (bcut), this routine returns 
c   parameters defining the MTS (Minimum Truncation Set) of basis vector
c   indices; i.e. the minimum and maximum values of each index within
c   the set of "significant" basis vectors. The routine also returns the 
c   number of vectors qualifying as "significant" in nbas.
c
c   If the value of nmts is negative upon entry, the routine will also
c   count the number of vectors defined by the MTS parameters and return
c   it in nmts. In general, nmts need not equal nbas; i.e., the MTS still 
c   contains some insignificant vectors.
c
c   This routine replaces the function of program TNLL in EPRLL versions
c   through 1.3. It is intended to be called primarily by routine EPRBL.
c
c   Written by David Budil, 18 May 1993, Cornell Dept. of Chemistry
c   Modified by AP 17-FEB-94 for EPRSR calculation.
c
c   Uses:
c     ipar.f
c
c   Includes:
c     maxl.inc 
c     stddim.inc
c     eprdat.inc
c     indexl.inc
c     srsflg.inc
c----------------------------------------------------------------------
c
      subroutine mtss( basis,nbasis,bcut,maxle,minle,maxlo,minlo,
     #                 maxm,minm,maxk,mink,maxj,minj,maxn,minn,
     #                 maxi,mini,maxpi,minpi,nbas,nmts )
c
      include 'stddim.inc'
c
c     implicit none
      double precision bcut,basis(2,ndim)
      integer nbasis,maxle,minle,maxlo,minlo,maxm,minm,maxk,mink,
     #        maxj,minj,maxn,minn,maxi,mini,maxpi,minpi,nbas,nmts
c
      include 'maxl.inc'
      include 'indexl.inc'
      include 'eprdat.inc'
c
      include 'srsflg.inc'
c
      integer lr,mr,kr,jr,nr,ir,ipnr,imr,iqnr,ipsr,iqsr
      integer mrmx,krmx,nrmn,nrmx,irmx,ipnrmn,ipnrmx,imrmn,imrmx,
     #        iqnrmn,iqnrmx,ipsrmn,ipsrmx,iqsrmn,iqsrmx,
     #        nrmax,ipnrmax
      integer i,n
      double precision zero
      parameter (zero=0.d0)
c
      integer ipar
      external ipar
c
c----------------------------------------------------------------------
c
      minle=mxlval+1
      maxle=-mxlval-1
      minlo=mxlval+1
      maxlo=-mxlval-1
      minm=minle
      maxm=maxle
      mink=minle
      maxk=maxle
      minj=minle
      maxj=maxle
      minn=minle
      maxn=maxle
      mini=minle
      maxi=maxle
      minpi=in2+1
      maxpi=-in2-1
c
      nbas=0
c
      do 10 i=1,nbasis
         if (basis(2,i).ge.bcut) then
            nbas=nbas+1
c
c     write(*,1000) l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),pi1(i)
c1000 format(2x,7(i2,','),f12.6)
            n=l1(i)
            if (ipar(n).eq.1) then
              if (n.gt.maxle) maxle=n
              if (n.lt.minle) minle=n
            else
              if (n.gt.maxlo) maxlo=n
              if (n.lt.minlo) minlo=n
            end if
c
            n=m1(i)
            if (n.ge.maxm) maxm=n
            if (n.le.minm) minm=n
c
            n=k1(i)
            if (n.ge.maxk) maxk=n
            if (n.le.mink) mink=n
c
            n=j1(i)
            if (n.ge.maxj) maxj=n
            if (n.le.minj) minj=n
c
            n=n1(i)
            if (n.ge.maxn) maxn=n
            if (n.le.minn) minn=n
c
            n=i1(i)
            if (n.ge.maxi) maxi=n
            if (n.le.mini) mini=n
c
            n=pi1(i)
            if (n.ge.maxpi) maxpi=n
            if (n.le.minpi) minpi=n
         end if
 10   continue
c
c----------------------------------------------------------------------
c     Now count the number of elements in the MTS if needed
c----------------------------------------------------------------------
c
      if (nmts.lt.0) then
         nmts=0
c
c     *** loop over lr ***
c
      do 100 lr=0,maxle,1
        if((ipar(lr).ne.1).and.((lr.gt.maxlo).or.(lr.lt.minlo))) 
     #  go to 100
c
c     *** loop over mr ***
c
       if(lkmtrc.eq.1) then
           mrmx=min(maxm,lr,lemx-lr+lkmdff)
       else
           mrmx=min(maxm,lr)
       end if
       do 110 mr=0,mrmx,1
c
c     *** loop over kr ***
c
        if(lkmtrc.eq.1) then
           krmx=min(maxk,lr,lemx-lr-mr+lkmdff)
        else
           krmx=min(maxk,lr)
        end if
        do 120 kr=0,krmx,kdelta
        if (kr.eq.0.and.ipar(lr).ne.1) go to 120
c 
c     *** loop over jr ***
c
         do 122 jr=0,maxj,jdelta
c
c     *** loop over nr ***
c
          nrmax=max(iabs(maxn),iabs(minn))
          nrmx=min(nrmax,jr)
          if (mr.eq.0) then
             nrmn=0
          else
             nrmn=-nrmx
          end if
          do 124 nr=nrmn,nrmx,1
c
c     *** loop over ir ***
c
        irmx=min(maxi,jr)
        do 125 ir=0,irmx
        if (ir.eq.0.and.ipar(jr).ne.1) go to 125
c
c     *** loop over ipnr ***
c
           ipnrmax=max(iabs(maxpi),iabs(minpi))
           ipnrmx=min(ipnrmax,in2)
           if (mr.eq.0.and.nr.eq.0) then
              ipnrmn=0
           else
              ipnrmn=-ipnrmx
           end if
           do 126 ipnr=ipnrmn,ipnrmx,1
              if (psi.eq.zero.and.(mr+nr).ne.ipnr) go to 126
c
c     *** loop over im ***
c
              imrmx=1
              if (hfdflg) then
                 imrmn=-1
              else
                 imrmn=1
              end if
c
            do 127 imr=imrmn,imrmx,2
               if (mr.eq.0.and.nr.eq.0.and.ipnr.eq.0.and.
     #             ipar(lr+jr).ne.imr) go to 127
c
c     *** loop over iqnr ***
c
             iqnrmx=in2-iabs(ipnr)
             iqnrmn=-iqnrmx
             do 128 iqnr=iqnrmn,iqnrmx,2
c
c     *** loop over ipsr ***
c
              ipsrmx=1
              if (hfdflg) then
                 ipsrmn=-1
              else
                 ipsrmn=1
              end if
              do 130 ipsr=ipsrmn,ipsrmx,1
c
c     *** loop over iqsr ***
c
               iqsrmx=1-iabs(ipsr)
               iqsrmn=-iqsrmx
               do 140 iqsr=iqsrmn,iqsrmx,2
c
                 nmts=nmts+1
c
c----------------------------------------------------------------------
c     end loop over rows                                      
c----------------------------------------------------------------------
c
 140           continue
 130          continue
 128         continue
 127        continue
 126       continue
 125       continue
 124      continue
 122     continue
 120    continue
 110   continue
 100  continue
c
c----------------------------------------------------------------------
c
      end if
c     
      return
      end









